package yrpg.bamp.basedata.webapi;

import kd.bos.context.RequestContext;
import kd.bos.dataentity.OperateOption;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.operate.result.OperateErrorInfo;
import kd.bos.entity.operate.result.OperationResult;
import kd.bos.entity.validate.ValidateResult;
import kd.bos.openapi.common.custom.annotation.ApiController;
import kd.bos.openapi.common.custom.annotation.ApiMapping;
import kd.bos.openapi.common.custom.annotation.ApiParam;
import kd.bos.openapi.common.custom.annotation.ApiPostMapping;
import kd.bos.openapi.common.result.CustomApiResult;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.QueryServiceHelper;
import kd.bos.servicehelper.operation.OperationServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import yrpg.bamp.basedata.webapi.pojo.GoodsDTO;
import yrpg.bamp.basedata.webapi.pojo.GoodsUnitConversionDTO;


import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

import static yrpg.bamp.basedata.webapi.NewCustomerSaveApiPlugin.handleBooleanFields;


@ApiController(value = "goods", desc = "商品")
@ApiMapping("/goods")
public class GoodsSaveApisPlugin {
    @ApiPostMapping(value = "/saveGoodsInformation", desc = "主数据商品信息")
    public CustomApiResult<Object> saveGoodsInformation(
            @ApiParam(value = "编码", required = true, message = "商品编码不能为空") String number,
            @ApiParam(value = "名称", required = true, message = "商品名称不能为空") String name,
            @ApiParam(value = "产地") String origination,
            @ApiParam(value = "规格组合名", required = true, message = "商品规格组合名不能为空") String goods_specs,
            @ApiParam(value = "剂型") String dosage_form,
            @ApiParam(value = "最小计量单位编码", required = true, message = "商品最小单位编码不能为空") String base_unit_number,
            @ApiParam(value = "最小计量单位名称", required = true, message = "商品最小单位名称不能为空") String base_unit_name,
            @ApiParam(value = "生产厂家") String manufacturer,
            @ApiParam(value = "批准文号") String approval_no,
            @ApiParam(value = "商品条码") String goods_barcode,
            @ApiParam(value = "商品状态") String medicine_status,
            @ApiParam(value = "包装规格") String pack_spec,
            @ApiParam(value = "销售规格") String sales_spec,
            @ApiParam(value = "产品规格") String goods_spec,
            @ApiParam(value = "商品类别1") String goods_category1,
            @ApiParam(value = "商品类别2") String goods_category2,
            @ApiParam(value = "商品类别3") String goods_category3,
            @ApiParam(value = "备注") String notes,
            @ApiParam(value = "有效期") String expiry_char,
            @ApiParam(value = "国家集采品种") String country_variety,
            @ApiParam(value = "JDE编码") String goods_jde_number,
            @ApiParam(value = "单位转换") List<GoodsUnitConversionDTO> unit_conversion,
            @ApiParam(value = "使用状态", required = true, message = "商品使用状态不能为空") String enable,
            @ApiParam(value = "操作类型", required = true, message = "操作类型不能为空") String operationType
    ) {
        DynamicObject goods = BusinessDataServiceHelper.loadSingle("yrpg_mkt_goods",
                QFilter.of("number = ?", number).toArray());
        if (operationType.equals("3") || operationType.equals("4")) {
            if (goods != null) {
                String ableStatus = operationType.equals("3") ? "enable" : "disable";
                OperationServiceHelper.executeOperate(ableStatus, "yrpg_mkt_goods", new DynamicObject[]{goods}, OperateOption.create());
                return CustomApiResult.success("success");
            } else {
                return CustomApiResult.fail("", "未查询到该商品,商品编码为:{" + number + "}");
            }
        }
        for (GoodsUnitConversionDTO goodsUnit : unit_conversion) {
            if (goodsUnit.getNumber() == null || goodsUnit.getNumber().trim().isEmpty()) {
                return CustomApiResult.fail("", "单位转换中的商品单位转换编码不能为空");
            }
            if (goodsUnit.getAssist_unit_number() == null || goodsUnit.getAssist_unit_number().trim().isEmpty()) {
                return CustomApiResult.fail("", "单位转换中的商品单位转换辅助计量单位编码不能为空");
            }
            if (goodsUnit.getAssist_unit_name() == null || goodsUnit.getAssist_unit_name().trim().isEmpty()) {
                return CustomApiResult.fail("", "单位转换中的商品单位转换辅助计量单位名称不能为空");
            }
            if (goodsUnit.getUnit_number() == null || goodsUnit.getUnit_number().trim().isEmpty()) {
                return CustomApiResult.fail("", "单位转换中的商品单位转换最小计量单位编码不能为空");
            }
            if (goodsUnit.getUnit_name() == null || goodsUnit.getUnit_name().trim().isEmpty()) {
                return CustomApiResult.fail("", "单位转换中的商品单位转换最小计量单位名称不能为空");
            }
            if (goodsUnit.getMember_number() == null || goodsUnit.getMember_number().trim().isEmpty()) {
                return CustomApiResult.fail("", "单位转换中的商品单位转换分子不能为空");
            }
            if (goodsUnit.getDenominator_number() == null || goodsUnit.getDenominator_number().trim().isEmpty()) {
                return CustomApiResult.fail("", "单位转换中的商品单位转换分母不能为空");
            }
        }
        DynamicObject user = BusinessDataServiceHelper.loadSingle(RequestContext.get().getCurrUserId(), "bos_user");
        DynamicObject unit = BusinessDataServiceHelper.loadSingle("yrpg_sys_assinfo",
                QFilter.of("group.number=? and number=?", "01", base_unit_number).toArray());
        if (unit == null) {
            unit = createUnit(base_unit_number, base_unit_name);
        }
        if (goods != null) {
            goods.set("name", name);
            goods.set("yrpg_goods_alias", name);
            goods.set("yrpg_origination", origination);
            goods.set("yrpg_goods_specs", goods_specs);
            goods.set("yrpg_dosage_form", dosage_form);
            goods.set("yrpg_manufacturer", manufacturer);
            goods.set("yrpg_approval_no", approval_no);
            goods.set("yrpg_goods_category1", goods_category1);
            goods.set("yrpg_goods_category2", goods_category2);
            goods.set("yrpg_goods_category3", goods_category3);
            goods.set("yrpg_goods_barcode", goods_barcode);
            goods.set("yrpg_medicine_status", medicine_status);
            goods.set("yrpg_pack_spec", pack_spec);
            goods.set("yrpg_sales_spec", sales_spec);
            goods.set("yrpg_goods_spec", goods_spec);
            goods.set("yrpg_notes", notes);
            goods.set("yrpg_expiry_char", expiry_char);
            goods.set("yrpg_country_variety", country_variety);
            goods.set("yrpg_baseunit", unit);
        } else {
            goods = BusinessDataServiceHelper.newDynamicObject("yrpg_mkt_goods");
            goods.set("number", number);
            goods.set("name", name);
            goods.set("yrpg_goods_alias", name);
            goods.set("yrpg_origination", origination);
            goods.set("yrpg_goods_specs", goods_specs);
            goods.set("yrpg_dosage_form", dosage_form);
            goods.set("yrpg_manufacturer", manufacturer);
            goods.set("yrpg_approval_no", approval_no);
            goods.set("yrpg_goods_category1", goods_category1);
            goods.set("yrpg_goods_category2", goods_category2);
            goods.set("yrpg_goods_category3", goods_category3);
            goods.set("yrpg_goods_barcode", goods_barcode);
            goods.set("yrpg_medicine_status", medicine_status);
            goods.set("yrpg_pack_spec", pack_spec);
            goods.set("yrpg_sales_spec", sales_spec);
            goods.set("yrpg_goods_spec", goods_spec);
            goods.set("yrpg_notes", notes);
            goods.set("yrpg_expiry_char", expiry_char);
            goods.set("yrpg_country_variety", country_variety);
            goods.set("yrpg_baseunit", unit);
            goods.set("yrpg_goods_jde_number", goods_jde_number);
            goods.set("creator", user);
            goods.set("yrpg_auditor", user);
            goods.set("yrpg_auditdate", new Date());
            goods.set("enable", enable);
            goods.set("status", "C");
        }
        OperationResult result = SaveServiceHelper.saveOperate("yrpg_mkt_goods", new DynamicObject[]{goods}, OperateOption.create());
        if (result.isSuccess()) {
            DynamicObject full_goods = BusinessDataServiceHelper.loadSingle(result.getSuccessPkIds().get(0), "yrpg_mkt_goods");
            for (GoodsUnitConversionDTO goodsUnit : unit_conversion) {
                DynamicObject unitConversion = BusinessDataServiceHelper.loadSingle("yrpg_mkt_goods_unit",
                        QFilter.of("yrpg_master_data_id = ?", goodsUnit.getNumber()).toArray()
                );
                DynamicObject assist = BusinessDataServiceHelper.loadSingle("yrpg_sys_assinfo",
                        QFilter.of("group.number=? and number=?", "01", goodsUnit.getAssist_unit_number()).toArray());
                DynamicObject major = BusinessDataServiceHelper.loadSingle("yrpg_sys_assinfo",
                        QFilter.of("group.number=? and number=?", "01", goodsUnit.getUnit_number()).toArray());
                if (assist == null) {
                    assist = createUnit(goodsUnit.getAssist_unit_number(), goodsUnit.getAssist_unit_name());
                }
                if (major == null) {
                    major = createUnit(goodsUnit.getUnit_number(), goodsUnit.getUnit_name());
                }
                if (unitConversion != null) {
                    unitConversion.set("yrpg_member_number", goodsUnit.getMember_number());
                    unitConversion.set("yrpg_denominator_number", goodsUnit.getDenominator_number());
                    unitConversion.set("yrpg_assist_unit", assist);
                    unitConversion.set("yrpg_unit", major);
                    unitConversion.set("yrpg_is_sku_unit", handleBooleanFields(goodsUnit.getIs_sku_unit()));
                } else {
                    unitConversion = BusinessDataServiceHelper.newDynamicObject("yrpg_mkt_goods_unit");
                    unitConversion.set("yrpg_master_data_id", goodsUnit.getNumber());
                    unitConversion.set("yrpg_goods_id", full_goods);
                    unitConversion.set("yrpg_member_number", goodsUnit.getMember_number());
                    unitConversion.set("yrpg_denominator_number", goodsUnit.getDenominator_number());
                    unitConversion.set("yrpg_assist_unit", assist);
                    unitConversion.set("yrpg_unit", major);
                    unitConversion.set("yrpg_is_sku_unit", handleBooleanFields(goodsUnit.getIs_sku_unit()));
                    unitConversion.set("creator", user);
                    unitConversion.set("enable", "1");
                    unitConversion.set("status", "C");
                }
                SaveServiceHelper.saveOperate("yrpg_mkt_goods_unit", new DynamicObject[]{unitConversion}, OperateOption.create());
            }
            return CustomApiResult.success("success");
        } else {
            List<ValidateResult> validateErrors = result.getValidateResult().getValidateErrors();
            if (!validateErrors.isEmpty()) {
                List<OperateErrorInfo> allErrorInfo = validateErrors.get(0).getAllErrorInfo();
                if (!allErrorInfo.isEmpty()) {
                    StringBuilder message = new StringBuilder();
                    for (OperateErrorInfo operateErrorInfo : allErrorInfo) {
                        message.append(operateErrorInfo.getMessage());
                    }
                    return CustomApiResult.fail("", message.toString());
                } else {
                    return CustomApiResult.fail("", result.getMessage());
                }
            } else {
                return CustomApiResult.fail("", result.getMessage());
            }
        }
    }

    @ApiPostMapping(value = "/batchSaveGoodsInformation", desc = "主数据商品信息")
    public CustomApiResult<Object> batchSaveGoodsInformation(
            @ApiParam(value = "商品数据", required = true, message = "商品数据不能为空") List<GoodsDTO> goodsList,
            @ApiParam(value = "操作类型", required = true, message = "操作类型不能为空") String operationType
    ) {
        if (goodsList.size() > 500) {
            return CustomApiResult.fail("", "请一次性处理500条以下的数据");
        }
        Map<String, Object> map = new HashMap<>();
        ArrayList<Map<String, Object>> resultList = new ArrayList<>();
        DynamicObject user = BusinessDataServiceHelper.loadSingle(RequestContext.get().getCurrUserId(), "bos_user");
        ArrayList<GoodsDTO> failedList = new ArrayList<>();
        ArrayList<DynamicObject> list = new ArrayList<>();
        a:
        for (GoodsDTO goodsDTO : goodsList) {
            if (goodsDTO.getUnit_conversion() != null) {
                for (GoodsUnitConversionDTO goodsUnitConversionDTO : goodsDTO.getUnit_conversion()) {
                    if (checkRequiredFields(goodsUnitConversionDTO)) {
                        Map<String, Object> goodsMap = new HashMap<>();
                        goodsMap.put("id", "");
                        goodsMap.put("number", goodsDTO.getNumber());
                        goodsMap.put("billStatus", false);
                        goodsMap.put("errors", "存在未填写完整的单位转换字段");
                        resultList.add(goodsMap);
                        failedList.add(goodsDTO);
                        continue a;
                    }
                }
            }
            DynamicObject goods = BusinessDataServiceHelper.loadSingle("yrpg_mkt_goods",
                    QFilter.of("number = ?", goodsDTO.getNumber()).toArray());
            DynamicObject unit = BusinessDataServiceHelper.loadSingle("yrpg_sys_assinfo",
                    QFilter.of("group.number=? and number=?", "01", goodsDTO.getBase_unit_number()).toArray());
            if (unit == null) {
                unit = createUnit(goodsDTO.getBase_unit_number(), goodsDTO.getBase_unit_name());
            }
            if (goods != null) {
                goods.set("name", goodsDTO.getName());
                goods.set("yrpg_goods_alias", goodsDTO.getName());
                goods.set("yrpg_origination", goodsDTO.getOrigination());
                goods.set("yrpg_goods_specs", goodsDTO.getGoods_specs());
                goods.set("yrpg_dosage_form", goodsDTO.getDosage_form());
                goods.set("yrpg_manufacturer", goodsDTO.getManufacturer());
                goods.set("yrpg_approval_no", goodsDTO.getApproval_no());
                goods.set("yrpg_goods_category1", goodsDTO.getGoods_category1());
                goods.set("yrpg_goods_category2", goodsDTO.getGoods_category2());
                goods.set("yrpg_goods_category3", goodsDTO.getGoods_category3());
                goods.set("yrpg_goods_barcode", goodsDTO.getGoods_barcode());
                goods.set("yrpg_medicine_status", goodsDTO.getMedicine_status());
                goods.set("yrpg_pack_spec", goodsDTO.getPack_spec());
                goods.set("yrpg_sales_spec", goodsDTO.getSales_spec());
                goods.set("yrpg_goods_spec", goodsDTO.getGoods_spec());
                goods.set("yrpg_goods_spec_abbr", goodsDTO.getGoods_spec_abbr());
                goods.set("yrpg_notes", goodsDTO.getNotes());
                goods.set("yrpg_expiry_char", goodsDTO.getExpiry_char());
                goods.set("yrpg_country_variety", goodsDTO.getCountry_variety());
                goods.set("yrpg_trade_code", goodsDTO.getTrade_code());
                goods.set("yrpg_baseunit", unit);
                goods.set("enable", goodsDTO.getEnable());
            } else {
                goods = BusinessDataServiceHelper.newDynamicObject("yrpg_mkt_goods");
                goods.set("number", goodsDTO.getNumber());
                goods.set("name", goodsDTO.getName());
                goods.set("yrpg_goods_alias", goodsDTO.getName());
                goods.set("yrpg_origination", goodsDTO.getOrigination());
                goods.set("yrpg_goods_specs", goodsDTO.getGoods_specs());
                goods.set("yrpg_dosage_form", goodsDTO.getDosage_form());
                goods.set("yrpg_manufacturer", goodsDTO.getManufacturer());
                goods.set("yrpg_approval_no", goodsDTO.getApproval_no());
                goods.set("yrpg_goods_category1", goodsDTO.getGoods_category1());
                goods.set("yrpg_goods_category2", goodsDTO.getGoods_category2());
                goods.set("yrpg_goods_category3", goodsDTO.getGoods_category3());
                goods.set("yrpg_goods_barcode", goodsDTO.getGoods_barcode());
                goods.set("yrpg_medicine_status", goodsDTO.getMedicine_status());
                goods.set("yrpg_pack_spec", goodsDTO.getPack_spec());
                goods.set("yrpg_sales_spec", goodsDTO.getSales_spec());
                goods.set("yrpg_goods_spec", goodsDTO.getGoods_spec());
                goods.set("yrpg_goods_spec_abbr", goodsDTO.getGoods_spec_abbr());
                goods.set("yrpg_notes", goodsDTO.getNotes());
                goods.set("yrpg_expiry_char", goodsDTO.getExpiry_char());
                goods.set("yrpg_country_variety", goodsDTO.getCountry_variety());
                goods.set("yrpg_baseunit", unit);
                goods.set("yrpg_goods_jde_number", goodsDTO.getGoods_jde_number());
                goods.set("yrpg_trade_code", goodsDTO.getTrade_code());
                goods.set("creator", user);
                goods.set("yrpg_auditor", user);
                goods.set("yrpg_auditdate", new Date());
                goods.set("enable", goodsDTO.getEnable());
                goods.set("status", "C");
            }
            list.add(goods);
        }
        OperationResult result = SaveServiceHelper.saveOperate("yrpg_mkt_goods", list.toArray(new DynamicObject[0]), OperateOption.create());
        if (result.isSuccess()) {
            DynamicObjectCollection successList = QueryServiceHelper.query("yrpg_mkt_goods", "id,number",
                    new QFilter("id", QCP.in, result.getSuccessPkIds()).toArray());
            List<String> numberList = successList.stream().map(o -> o.getString("number"))
                    .collect(Collectors.toList());
            for (DynamicObject goods : successList) {
                Map<String, Object> goodsMap = new HashMap<>();
                goodsMap.put("id", goods.get("id"));
                goodsMap.put("number", goods.get("number"));
                goodsMap.put("billStatus", true);
                goodsMap.put("errors", "");
                resultList.add(goodsMap);
            }
            map.put("successCount", String.valueOf(result.getSuccessPkIds().size()));
            map.put("failCount", String.valueOf(failedList.size()));
            map.put("totalCount", String.valueOf(result.getSuccessPkIds().size() + failedList.size()));
            map.put("result", resultList);

            goodsList.removeAll(failedList);
            ArrayList<DynamicObject> unitList = new ArrayList<>();
            for (GoodsDTO goodsDTO : goodsList) {
                if (numberList.contains(goodsDTO.getNumber())) {
                    if (goodsDTO.getUnit_conversion() != null) {
                        DynamicObject full_goods = BusinessDataServiceHelper.loadSingle("yrpg_mkt_goods",
                                QFilter.of("number = ?", goodsDTO.getNumber()).toArray());
                        for (GoodsUnitConversionDTO goodsUnit : goodsDTO.getUnit_conversion()) {
                            DynamicObject unitConversion = BusinessDataServiceHelper.loadSingle("yrpg_mkt_goods_unit",
                                    QFilter.of("yrpg_master_data_id = ?", goodsUnit.getNumber()).toArray());
                            DynamicObject assist = BusinessDataServiceHelper.loadSingle("yrpg_sys_assinfo",
                                    QFilter.of("group.number=? and number=?", "01", goodsUnit.getAssist_unit_number()).toArray());
                            DynamicObject major = BusinessDataServiceHelper.loadSingle("yrpg_sys_assinfo",
                                    QFilter.of("group.number=? and number=?", "01", goodsUnit.getUnit_number()).toArray());
                            if (assist == null) {
                                assist = createUnit(goodsUnit.getAssist_unit_number(), goodsUnit.getAssist_unit_name());
                            }
                            if (major == null) {
                                major = createUnit(goodsUnit.getUnit_number(), goodsUnit.getUnit_name());
                            }
                            if (unitConversion != null) {
                                unitConversion.set("yrpg_member_number", goodsUnit.getMember_number());
                                unitConversion.set("yrpg_denominator_number", goodsUnit.getDenominator_number());
                                unitConversion.set("yrpg_assist_unit", assist);
                                unitConversion.set("yrpg_unit", major);
                                unitConversion.set("yrpg_is_sku_unit", handleBooleanFields(goodsUnit.getIs_sku_unit()));
                            } else {
                                unitConversion = BusinessDataServiceHelper.newDynamicObject("yrpg_mkt_goods_unit");
                                unitConversion.set("yrpg_master_data_id", goodsUnit.getNumber());
                                unitConversion.set("yrpg_goods_id", full_goods);
                                unitConversion.set("yrpg_member_number", goodsUnit.getMember_number());
                                unitConversion.set("yrpg_denominator_number", goodsUnit.getDenominator_number());
                                unitConversion.set("yrpg_assist_unit", assist);
                                unitConversion.set("yrpg_unit", major);
                                unitConversion.set("yrpg_is_sku_unit", handleBooleanFields(goodsUnit.getIs_sku_unit()));
                                unitConversion.set("creator", user);
                                unitConversion.set("enable", "1");
                                unitConversion.set("status", "C");
                            }
                            unitList.add(unitConversion);
                        }
                    }
                }
            }
            SaveServiceHelper.saveOperate("yrpg_mkt_goods_unit", unitList.toArray(new DynamicObject[0]), OperateOption.create());
            return CustomApiResult.success(map);
        } else {
            DynamicObjectCollection successList = QueryServiceHelper.query("yrpg_mkt_goods", "id,number",
                    new QFilter("id", QCP.in, result.getSuccessPkIds()).toArray());
            List<String> numberList = successList.stream().map(o -> o.getString("number"))
                    .collect(Collectors.toList());
            for (DynamicObject goods : successList) {
                Map<String, Object> goodsMap = new HashMap<>();
                goodsMap.put("id", goods.get("id"));
                goodsMap.put("number", goods.get("number"));
                goodsMap.put("billStatus", true);
                goodsMap.put("errors", "");
                resultList.add(goodsMap);
            }
            map.put("successCount", String.valueOf(result.getSuccessPkIds().size()));
            map.put("failCount", String.valueOf(result.getValidateResult().getErrorDataIndexs().size() + failedList.size()));
            map.put("totalCount", String.valueOf(result.getSuccessPkIds().size() + result.getValidateResult().getErrorDataIndexs().size() + failedList.size()));
            List<OperateErrorInfo> allErrorInfo = result.getValidateResult().getValidateErrors().get(0).getAllErrorInfo();
            if (allErrorInfo.size() == 1 && !allErrorInfo.get(0).getMessage().contains("：")) {
                Map<String, Object> goodsMap = new HashMap<>();
                goodsMap.put("id", "");
                goodsMap.put("number", result.getBillNos().toString().replace("}", "").split("=")[1]);
                goodsMap.put("billStatus", false);
                goodsMap.put("errors", allErrorInfo.get(0).getMessage());
                resultList.add(goodsMap);
            } else {
                for (OperateErrorInfo operateErrorInfo : allErrorInfo) {
                    Map<String, Object> goodsMap = new HashMap<>();
                    goodsMap.put("id", "");
                    String[] split = operateErrorInfo.getMessage().split("：");
                    goodsMap.put("number", split[0]);
                    goodsMap.put("billStatus", false);
                    goodsMap.put("errors", split[1]);
                    resultList.add(goodsMap);
                }
                Map<String, Long> collect = resultList.stream()
                        .filter(o -> o.get("billStatus").equals(false))
                        .map(o -> o.get("number").toString())
                        .collect(Collectors.toList()).stream()
                        .collect(Collectors.groupingBy(Function.identity(), Collectors.counting()));
                List<String> duplicates = collect.entrySet().stream()
                        .filter(entry -> entry.getValue() > 1)
                        .map(Map.Entry::getKey)
                        .collect(Collectors.toList());
                List<Map<String, Object>> last = resultList.stream()
                        .filter(o -> o.get("billStatus").equals(false))
                        .filter(o -> duplicates.contains(o.get("number").toString()))
                        .collect(Collectors.toList());
                resultList.removeAll(last);
                for (String duplicate : duplicates) {
                    StringBuilder msg = new StringBuilder();
                    for (Map<String, Object> err : last) {
                        if (err.get("number").equals(duplicate)) {
                            msg.append(err.get("errors"));
                        }
                    }
                    Map<String, Object> goodsMap = new HashMap<>();
                    goodsMap.put("id", "");
                    goodsMap.put("number", duplicate);
                    goodsMap.put("billStatus", false);
                    goodsMap.put("errors", msg);
                    resultList.add(goodsMap);
                }
            }
            map.put("result", resultList);

            goodsList.removeAll(failedList);
            ArrayList<DynamicObject> unitList = new ArrayList<>();
            for (GoodsDTO goodsDTO : goodsList) {
                if (numberList.contains(goodsDTO.getNumber())) {
                    if (goodsDTO.getUnit_conversion() != null) {
                        DynamicObject full_goods = BusinessDataServiceHelper.loadSingle("yrpg_mkt_goods",
                                QFilter.of("number = ?", goodsDTO.getNumber()).toArray());
                        for (GoodsUnitConversionDTO goodsUnit : goodsDTO.getUnit_conversion()) {
                            DynamicObject unitConversion = BusinessDataServiceHelper.loadSingle("yrpg_mkt_goods_unit",
                                    QFilter.of("yrpg_master_data_id = ?", goodsUnit.getNumber()).toArray());
                            DynamicObject assist = BusinessDataServiceHelper.loadSingle("yrpg_sys_assinfo",
                                    QFilter.of("group.number=? and number=?", "01", goodsUnit.getAssist_unit_number()).toArray());
                            DynamicObject major = BusinessDataServiceHelper.loadSingle("yrpg_sys_assinfo",
                                    QFilter.of("group.number=? and number=?", "01", goodsUnit.getUnit_number()).toArray());
                            if (assist == null) {
                                assist = createUnit(goodsUnit.getAssist_unit_number(), goodsUnit.getAssist_unit_name());
                            }
                            if (major == null) {
                                major = createUnit(goodsUnit.getUnit_number(), goodsUnit.getUnit_name());
                            }
                            if (unitConversion != null) {
                                unitConversion.set("member_number", goodsUnit.getMember_number());
                                unitConversion.set("yrpg_denominator_number", goodsUnit.getDenominator_number());
                                unitConversion.set("yrpg_assist_unit", assist);
                                unitConversion.set("yrpg_unit", major);
                                unitConversion.set("yrpg_is_sku_unit", handleBooleanFields(goodsUnit.getIs_sku_unit()));
                            } else {
                                unitConversion = BusinessDataServiceHelper.newDynamicObject("yrpg_mkt_goods_unit");
                                unitConversion.set("yrpg_master_data_id", goodsUnit.getNumber());
                                unitConversion.set("yrpg_goods_id", full_goods);
                                unitConversion.set("yrpg_member_number", goodsUnit.getMember_number());
                                unitConversion.set("yrpg_denominator_number", goodsUnit.getDenominator_number());
                                unitConversion.set("yrpg_assist_unit", assist);
                                unitConversion.set("yrpg_unit", major);
                                unitConversion.set("yrpg_is_sku_unit", handleBooleanFields(goodsUnit.getIs_sku_unit()));
                                unitConversion.set("creator", user);
                                unitConversion.set("enable", "1");
                                unitConversion.set("status", "C");
                            }
                            unitList.add(unitConversion);
                        }
                    }
                }
            }
            SaveServiceHelper.saveOperate("yrpg_mkt_goods_unit", unitList.toArray(new DynamicObject[0]), OperateOption.create());
            return CustomApiResult.success(map);
        }

    }

    private static DynamicObject createUnit(String number, String name) {
        DynamicObject group_one = BusinessDataServiceHelper.loadSingle("yrpg_sys_assinfo_group", QFilter.of("number=?", "01").toArray());
        DynamicObject unit_info = BusinessDataServiceHelper.newDynamicObject("yrpg_sys_assinfo");
        unit_info.set("number", number);
        unit_info.set("name", name);
        unit_info.set("group", group_one);
        unit_info.set("enable", "1");
        unit_info.set("status", "C");
        OperationResult result = SaveServiceHelper.saveOperate("yrpg_sys_assinfo", new DynamicObject[]{unit_info}, OperateOption.create());
        if (result.getSuccessPkIds().size()>0) {
            Object unit_id = result.getSuccessPkIds().get(0);
            return BusinessDataServiceHelper.loadSingle(unit_id, "yrpg_sys_assinfo");
        }
        return null;
    }

    private static Boolean checkRequiredFields(GoodsUnitConversionDTO goodsUnitConversionDTO) {
        return (goodsUnitConversionDTO.toString().contains("null") ||
                goodsUnitConversionDTO.toString().contains("''"));
    }
}
