package yrpg.bamp.basedata.webapi.masterdata;

import kd.bos.bill.IBillWebApiPlugin;
import kd.bos.context.RequestContext;
import kd.bos.dataentity.OperateOption;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.operate.result.OperateErrorInfo;
import kd.bos.entity.operate.result.OperationResult;
import kd.bos.entity.validate.ValidateResult;
import kd.bos.openapi.common.custom.annotation.ApiController;
import kd.bos.openapi.common.custom.annotation.ApiMapping;
import kd.bos.openapi.common.custom.annotation.ApiParam;
import kd.bos.openapi.common.custom.annotation.ApiPostMapping;
import kd.bos.openapi.common.result.CustomApiResult;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.QueryServiceHelper;
import kd.bos.servicehelper.operation.OperationServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import yrpg.bamp.basedata.webapi.pojo.CustomerDTO;


import java.util.Date;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

@ApiController(value = "customer", desc = "客户")
@ApiMapping("/customer")
public class CustomerSaveApiPlugin implements IBillWebApiPlugin {

    private static final DynamicObject yzj = BusinessDataServiceHelper.loadSingle(100000L, "bos_org");

    @ApiPostMapping(value = "/saveCustomerInformation", desc = "主数据客户信息")
    public CustomApiResult<Object> saveCustomerInformation(
            @ApiParam(value = "编码", required = true, message = "客户编码不能为空") String number,
            @ApiParam(value = "名称", required = true, message = "客户名称不能为空") String name,
            @ApiParam(value = "客户类型", required = true, message = "客户类型不能为空") String customer_class,
            @ApiParam(value = "省份名称") String province,
            @ApiParam(value = "城市名称") String city,
            @ApiParam(value = "区县名称") String district,
            @ApiParam(value = "曾用名") String previous_name,
            @ApiParam(value = "经度") String longitude,
            @ApiParam(value = "纬度") String latitude,
            @ApiParam(value = "统一社会信用代码") String uscc,
            @ApiParam(value = "经营许可证登记号") String business_reg_number,
            @ApiParam(value = "经营范围") String business_scope,
            @ApiParam(value = "营业期限") String business_term,
            @ApiParam(value = "法定代表人") String artificial_person,
            @ApiParam(value = "工商登记状态") String register_status,
            @ApiParam(value = "电话") String phone,
            @ApiParam(value = "是否医保医院(门店)") String is_insurance,
            @ApiParam(value = "是否分院") String is_branch,
            @ApiParam(value = "医院组合名称") String hospital_coname,
            @ApiParam(value = "客户等级") String customer_level,
            @ApiParam(value = "客户等别") String customer_grade,
            @ApiParam(value = "医保编码") String medicare_code,
            @ApiParam(value = "医院类型") String hospital_type,
            @ApiParam(value = "盈利类型") String profit_type,
            @ApiParam(value = "是否医联体") String is_medical_joined,
            @ApiParam(value = "是否民营医疗集团") String is_private_medicalgroup,
            @ApiParam(value = "经营方式") String operate_mode,
            @ApiParam(value = "药店属性") String pharmacy_attribute,
            @ApiParam(value = "商圈类型") String business_distinct_type,
            @ApiParam(value = "地域属性") String region_attribute,
            @ApiParam(value = "是否为o2o店") String is_o2o_store,
            @ApiParam(value = "是否医保双通道") String is_double_insurance,
            @ApiParam(value = "是否DTP药房") String is_dtp_store,
            @ApiParam(value = "全品类门店潜力等级") String store_potential,
            @ApiParam(value = "一级分类") String first_class,
            @ApiParam(value = "二级分类") String second_class,
            @ApiParam(value = "三级分类") String third_class,

            @ApiParam(value = "注册地址") String registered_address,
            @ApiParam(value = "收货地址组合名称") String shipping_address,

            @ApiParam(value = "总院行业库id") String general_hospital_id,
            @ApiParam(value = "总院主数据编码") String general_hospital_number,
            @ApiParam(value = "总院名称") String general_hospital_name,
            @ApiParam(value = "行业库id") String industry_id,
            @ApiParam(value = "上级行业库id") String higher_industry_id,
            @ApiParam(value = "上级主数据编码") String master_head_customer_id,
            @ApiParam(value = "上级名称") String higher_name,
            @ApiParam(value = "使用状态", required = true, message = "客户使用状态不能为空") String enable,
            @ApiParam(value = "操作类型", required = true, message = "操作类型不能为空") String operationType
    ) {
        DynamicObject customer = BusinessDataServiceHelper.loadSingle("rm_mkt_custom",
                QFilter.of("number = ?", number).toArray());
        if (operationType.equals("3") || operationType.equals("4")) {
            if (customer != null) {
                String ableStatus = operationType.equals("3") ? "enable" : "disable";
                OperationServiceHelper.executeOperate(ableStatus, "bos_user", new DynamicObject[]{customer}, OperateOption.create());
                return CustomApiResult.success("success");
            } else {
                return CustomApiResult.fail("", "未查询到该客户,客户编码为:{" + number + "}");
            }
        }
        DynamicObject user = BusinessDataServiceHelper.loadSingle(RequestContext.get().getCurrUserId(), "bos_user");
        DynamicObject firstClass = BusinessDataServiceHelper.loadSingle("rm_mkt_hospital_type", "id,number", QFilter.of("number=?", first_class).toArray());
        DynamicObject secondClass = BusinessDataServiceHelper.loadSingle("rm_mkt_hospital_type", "id,number", QFilter.of("number=?", second_class).toArray());
        DynamicObject thirdClass = BusinessDataServiceHelper.loadSingle("rm_mkt_hospital_type", "id,number", QFilter.of("number=?", third_class).toArray());
        if (customer != null) {
            customer.set("name", name);
            customer.set("customer_class", customer_class);
            customer.set("province", province);
            customer.set("city", city);
            customer.set("district", district);
            customer.set("previous_name", previous_name);
            customer.set("longitude", longitude);
            customer.set("latitude", latitude);
            customer.set("uscc", uscc);
            customer.set("business_reg_number", business_reg_number);
            customer.set("business_scope", business_scope);
            customer.set("business_term", business_term);
            customer.set("artificial_person", artificial_person);
            customer.set("register_status", register_status);
            customer.set("phone", phone);
            customer.set("is_insurance", handleBooleanFields(is_insurance));
            customer.set("is_branch", handleBooleanFields(is_branch));
            customer.set("hospital_coname", hospital_coname);
            customer.set("customer_level", customer_level);
            customer.set("customer_grade", customer_grade);
            customer.set("medicare_code", medicare_code);
            customer.set("hospital_type", hospital_type);
            customer.set("profit_type", profit_type);
            customer.set("is_medical_joined", handleBooleanFields(is_medical_joined));
            customer.set("is_private_medicalgroup", handleBooleanFields(is_private_medicalgroup));
            customer.set("operate_mode", operate_mode);
            customer.set("pharmacy_attribute", pharmacy_attribute);
            customer.set("business_distinct_type", business_distinct_type);
            customer.set("region_attribute", region_attribute);
            customer.set("is_o2o_store", handleBooleanFields(is_o2o_store));
            customer.set("is_double_insurance", handleBooleanFields(is_double_insurance));
            customer.set("is_dtp_store", handleBooleanFields(is_dtp_store));
            customer.set("store_potential", store_potential);
            customer.set("first_class", firstClass);
            customer.set("second_class", secondClass);
            customer.set("third_class", thirdClass);

            customer.set("general_hospital_id", general_hospital_id);
            customer.set("general_hospital_number", general_hospital_number);
            customer.set("general_hospital_name", general_hospital_name);
            customer.set("industry_id", industry_id);
            customer.set("higher_industry_id", higher_industry_id);
            customer.set("master_head_customer_id", master_head_customer_id);
            customer.set("higher_name", higher_name);

            DynamicObjectCollection customerAddress = customer.getDynamicObjectCollection("customer_address");
            customerAddress.clear();
            if (shipping_address != null && !shipping_address.isEmpty()) {
                DynamicObject ad = customerAddress.addNew();
                ad.set("address", shipping_address);
                ad.set("default_address", true);
            }
            if (registered_address != null && !registered_address.isEmpty()) {
                DynamicObject ad = customerAddress.addNew();
                ad.set("address", registered_address);
                ad.set("default_address", false);
            }

        } else {
            customer = BusinessDataServiceHelper.newDynamicObject("rm_mkt_custom");
            customer.set("number", number);
            customer.set("master_customer_id", number);
            customer.set("name", name);
            customer.set("customer_class", customer_class);
            customer.set("province", province);
            customer.set("city", city);
            customer.set("district", district);
            customer.set("previous_name", previous_name);
            customer.set("longitude", longitude);
            customer.set("latitude", latitude);
            customer.set("uscc", uscc);
            customer.set("business_reg_number", business_reg_number);
            customer.set("business_scope", business_scope);
            customer.set("business_term", business_term);
            customer.set("artificial_person", artificial_person);
            customer.set("register_status", register_status);
            customer.set("phone", phone);
            customer.set("is_insurance", handleBooleanFields(is_insurance));
            customer.set("is_branch", handleBooleanFields(is_branch));
            customer.set("hospital_coname", hospital_coname);
            customer.set("customer_level", customer_level);
            customer.set("customer_grade", customer_grade);
            customer.set("medicare_code", medicare_code);
            customer.set("hospital_type", hospital_type);
            customer.set("profit_type", profit_type);
            customer.set("is_medical_joined", handleBooleanFields(is_medical_joined));
            customer.set("is_private_medicalgroup", handleBooleanFields(is_private_medicalgroup));
            customer.set("operate_mode", operate_mode);
            customer.set("pharmacy_attribute", pharmacy_attribute);
            customer.set("business_distinct_type", business_distinct_type);
            customer.set("region_attribute", region_attribute);
            customer.set("is_o2o_store", handleBooleanFields(is_o2o_store));
            customer.set("is_double_insurance", handleBooleanFields(is_double_insurance));
            customer.set("is_dtp_store", handleBooleanFields(is_dtp_store));
            customer.set("store_potential", store_potential);
            customer.set("first_class", firstClass);
            customer.set("second_class", secondClass);
            customer.set("third_class", thirdClass);

            customer.set("general_hospital_id", general_hospital_id);
            customer.set("general_hospital_number", general_hospital_number);
            customer.set("general_hospital_name", general_hospital_name);
            customer.set("industry_id", industry_id);
            customer.set("higher_industry_id", higher_industry_id);
            customer.set("master_head_customer_id", master_head_customer_id);
            customer.set("higher_name", higher_name);

            customer.set("createorg", yzj);
            customer.set("useorg", yzj);
            customer.set("org", yzj);
            customer.set("creator", user);
            customer.set("auditor", user);
            customer.set("auditdate", new Date());

            customer.set("enable", enable);
            customer.set("status", "C");

            DynamicObjectCollection customerAddress = customer.getDynamicObjectCollection("customer_address");
            if (shipping_address != null && !shipping_address.isEmpty()) {
                DynamicObject ad = customerAddress.addNew();
                ad.set("address", shipping_address);
                ad.set("default_address", true);
            }
            if (registered_address != null && !registered_address.isEmpty()) {
                DynamicObject ad = customerAddress.addNew();
                ad.set("address", registered_address);
                ad.set("default_address", false);
            }
            DynamicObject sysInfo = BusinessDataServiceHelper.loadSingle("rm_sys_assinfo",
                    QFilter.of("group.number=? and number=?", "000003", "001").toArray());
            DynamicObjectCollection customerSrcSys = customer.getDynamicObjectCollection("customer_src_sys");
            DynamicObject sys = customerSrcSys.addNew();
            sys.set("src_sys_code", sysInfo);
        }
        OperationResult result = SaveServiceHelper.saveOperate("rm_mkt_custom", new DynamicObject[]{customer}, OperateOption.create());
        if (result.isSuccess()) {
            return CustomApiResult.success("success");
        } else {
            List<ValidateResult> validateErrors = result.getValidateResult().getValidateErrors();
            if (!validateErrors.isEmpty()) {
                List<OperateErrorInfo> allErrorInfo = validateErrors.get(0).getAllErrorInfo();
                if (!allErrorInfo.isEmpty()) {
                    StringBuilder message = new StringBuilder();
                    for (OperateErrorInfo operateErrorInfo : allErrorInfo) {
                        message.append(operateErrorInfo.getMessage());
                    }
                    return CustomApiResult.fail("", message.toString());
                } else {
                    return CustomApiResult.fail("", result.getMessage());
                }
            } else {
                return CustomApiResult.fail("", result.getMessage());
            }
        }
    }

    private static final Map<String, DynamicObject> hospClass =
            Arrays.stream(BusinessDataServiceHelper.load("rm_mkt_hospital_type", "id,number", null))
                    .collect(Collectors.toMap(o -> o.getString("number"), o -> o, (entity1, entity2) -> entity1));

    @ApiPostMapping(value = "/batchSaveCustomerInformation", desc = "主数据客户信息")
    public CustomApiResult<Object> batchSaveCustomerInformation(
            @ApiParam(value = "客户数据", required = true, message = "客户数据不能为空") List<CustomerDTO> cuseomerList,
            @ApiParam(value = "操作类型", required = true, message = "操作类型不能为空") String operationType
    ) {
        if (cuseomerList.size() > 500) {
            return CustomApiResult.fail("", "请一次性处理500条以下的数据");
        }
        DynamicObject user = BusinessDataServiceHelper.loadSingle(RequestContext.get().getCurrUserId(), "bos_user");
        DynamicObject sysInfo = BusinessDataServiceHelper.loadSingle("rm_sys_assinfo",
                QFilter.of("group.number=? and number=?", "000003", "001").toArray());
        Map<String, Object> map = new HashMap<>();
        ArrayList<Map<String, Object>> resultList = new ArrayList<>();
        ArrayList<DynamicObject> list = new ArrayList<>();
        for (CustomerDTO customerDTO : cuseomerList) {
            DynamicObject customer = BusinessDataServiceHelper.loadSingle("rm_mkt_custom",
                    QFilter.of("number = ?", customerDTO.getNumber()).toArray());
            if (customer != null) {
                customer.set("name", customerDTO.getName());
                customer.set("customer_class", customerDTO.getCustomer_class());
                customer.set("province", customerDTO.getProvince());
                customer.set("city", customerDTO.getCity());
                customer.set("district", customerDTO.getDistrict());
                customer.set("previous_name", customerDTO.getPrevious_name());
                customer.set("longitude", customerDTO.getLongitude());
                customer.set("latitude", customerDTO.getLatitude());
                customer.set("uscc", customerDTO.getUscc());
                customer.set("business_reg_number", customerDTO.getBusiness_reg_number());
                customer.set("business_scope", customerDTO.getBusiness_scope());
                customer.set("business_term", customerDTO.getBusiness_term());
                customer.set("artificial_person", customerDTO.getArtificial_person());
                customer.set("register_status", customerDTO.getRegister_status());
                customer.set("phone", customerDTO.getPhone());
                customer.set("is_insurance", handleBooleanFields(customerDTO.getIs_insurance()));
                customer.set("is_branch", handleBooleanFields(customerDTO.getIs_branch()));
                customer.set("hospital_coname", customerDTO.getHospital_coname());
                customer.set("customer_level", customerDTO.getCustomer_level());
                customer.set("customer_grade", customerDTO.getCustomer_grade());
                customer.set("medicare_code", customerDTO.getMedicare_code());
                customer.set("hospital_type", customerDTO.getHospital_type());
                customer.set("profit_type", customerDTO.getProfit_type());
                customer.set("is_medical_joined", handleBooleanFields(customerDTO.getIs_medical_joined()));
                customer.set("is_private_medicalgroup", handleBooleanFields(customerDTO.getIs_private_medicalgroup()));
                customer.set("operate_mode", customerDTO.getOperate_mode());
                customer.set("pharmacy_attribute", customerDTO.getPharmacy_attribute());
                customer.set("business_distinct_type", customerDTO.getBusiness_distinct_type());
                customer.set("region_attribute", customerDTO.getRegion_attribute());
                customer.set("is_o2o_store", handleBooleanFields(customerDTO.getIs_o2o_store()));
                customer.set("is_double_insurance", handleBooleanFields(customerDTO.getIs_double_insurance()));
                customer.set("is_dtp_store", handleBooleanFields(customerDTO.getIs_dtp_store()));
                customer.set("store_potential", customerDTO.getStore_potential());
                customer.set("first_class", hospClass.get(customerDTO.getFirst_class()));
                customer.set("second_class", hospClass.get(customerDTO.getSecond_class()));
                customer.set("third_class", hospClass.get(customerDTO.getThird_class()));

                customer.set("general_hospital_id", customerDTO.getGeneral_hospital_id());
                customer.set("general_hospital_number", customerDTO.getGeneral_hospital_number());
                customer.set("general_hospital_name", customerDTO.getGeneral_hospital_name());
                customer.set("industry_id", customerDTO.getIndustry_id());
                customer.set("higher_industry_id", customerDTO.getHigher_industry_id());
                customer.set("master_head_customer_id", customerDTO.getMaster_head_customer_id());
                customer.set("higher_name", customerDTO.getHigher_name());

                customer.set("enable", customerDTO.getEnable());

                DynamicObjectCollection customerAddress = customer.getDynamicObjectCollection("customer_address");
                customerAddress.clear();
                if (customerDTO.getShipping_address() != null && !customerDTO.getShipping_address().isEmpty()) {
                    DynamicObject ad = customerAddress.addNew();
                    ad.set("address", customerDTO.getShipping_address());
                    ad.set("default_address", true);
                }
                if (customerDTO.getRegistered_address() != null && !customerDTO.getRegistered_address().isEmpty()) {
                    DynamicObject ad = customerAddress.addNew();
                    ad.set("address", customerDTO.getRegistered_address());
                    ad.set("default_address", false);
                }

            } else {
                customer = BusinessDataServiceHelper.newDynamicObject("rm_mkt_custom");
                customer.set("number", customerDTO.getNumber());
                customer.set("master_customer_id", customerDTO.getNumber());
                customer.set("name", customerDTO.getName());
                customer.set("customer_class", customerDTO.getCustomer_class());
                customer.set("province", customerDTO.getProvince());
                customer.set("city", customerDTO.getCity());
                customer.set("district", customerDTO.getDistrict());
                customer.set("previous_name", customerDTO.getPrevious_name());
                customer.set("longitude", customerDTO.getLongitude());
                customer.set("latitude", customerDTO.getLatitude());
                customer.set("uscc", customerDTO.getUscc());
                customer.set("business_reg_number", customerDTO.getBusiness_reg_number());
                customer.set("business_scope", customerDTO.getBusiness_scope());
                customer.set("business_term", customerDTO.getBusiness_term());
                customer.set("artificial_person", customerDTO.getArtificial_person());
                customer.set("register_status", customerDTO.getRegister_status());
                customer.set("phone", customerDTO.getPhone());
                customer.set("is_insurance", handleBooleanFields(customerDTO.getIs_insurance()));
                customer.set("is_branch", handleBooleanFields(customerDTO.getIs_branch()));
                customer.set("hospital_coname", customerDTO.getHospital_coname());
                customer.set("customer_level", customerDTO.getCustomer_level());
                customer.set("customer_grade", customerDTO.getCustomer_grade());
                customer.set("medicare_code", customerDTO.getMedicare_code());
                customer.set("hospital_type", customerDTO.getHospital_type());
                customer.set("profit_type", customerDTO.getProfit_type());
                customer.set("is_medical_joined", handleBooleanFields(customerDTO.getIs_medical_joined()));
                customer.set("is_private_medicalgroup", handleBooleanFields(customerDTO.getIs_private_medicalgroup()));
                customer.set("operate_mode", customerDTO.getOperate_mode());
                customer.set("pharmacy_attribute", customerDTO.getPharmacy_attribute());
                customer.set("business_distinct_type", customerDTO.getBusiness_distinct_type());
                customer.set("region_attribute", customerDTO.getRegion_attribute());
                customer.set("is_o2o_store", handleBooleanFields(customerDTO.getIs_o2o_store()));
                customer.set("is_double_insurance", handleBooleanFields(customerDTO.getIs_double_insurance()));
                customer.set("is_dtp_store", handleBooleanFields(customerDTO.getIs_dtp_store()));
                customer.set("store_potential", customerDTO.getStore_potential());
                customer.set("first_class", hospClass.get(customerDTO.getFirst_class()));
                customer.set("second_class", hospClass.get(customerDTO.getSecond_class()));
                customer.set("third_class", hospClass.get(customerDTO.getThird_class()));

                customer.set("general_hospital_id", customerDTO.getGeneral_hospital_id());
                customer.set("general_hospital_number", customerDTO.getGeneral_hospital_number());
                customer.set("general_hospital_name", customerDTO.getGeneral_hospital_name());
                customer.set("industry_id", customerDTO.getIndustry_id());
                customer.set("higher_industry_id", customerDTO.getHigher_industry_id());
                customer.set("master_head_customer_id", customerDTO.getMaster_head_customer_id());
                customer.set("higher_name", customerDTO.getHigher_name());

                customer.set("createorg", yzj);
                customer.set("useorg", yzj);
                customer.set("org", yzj);
                customer.set("creator", user);
                customer.set("auditor", user);
                customer.set("auditdate", new Date());

                customer.set("enable", customerDTO.getEnable());
                customer.set("status", "C");

                DynamicObjectCollection customerAddress = customer.getDynamicObjectCollection("customer_address");
                if (customerDTO.getShipping_address() != null && !customerDTO.getShipping_address().isEmpty()) {
                    DynamicObject ad = customerAddress.addNew();
                    ad.set("address", customerDTO.getShipping_address());
                    ad.set("default_address", true);
                }
                if (customerDTO.getRegistered_address() != null && !customerDTO.getRegistered_address().isEmpty()) {
                    DynamicObject ad = customerAddress.addNew();
                    ad.set("address", customerDTO.getRegistered_address());
                    ad.set("default_address", false);
                }

                DynamicObjectCollection customerSrcSys = customer.getDynamicObjectCollection("customer_src_sys");
                DynamicObject sys = customerSrcSys.addNew();
                sys.set("src_sys_code", sysInfo);
            }
            list.add(customer);
        }
        OperationResult result = SaveServiceHelper.saveOperate("rm_mkt_custom", list.toArray(new DynamicObject[0]), OperateOption.create());
        if (result.isSuccess()) {
            DynamicObjectCollection successList = QueryServiceHelper.query("rm_mkt_custom", "id,number",
                    new QFilter("id", QCP.in, result.getSuccessPkIds()).toArray());
            for (DynamicObject customer : successList) {
                Map<String, Object> customerMap = new HashMap<>();
                customerMap.put("id", customer.get("id"));
                customerMap.put("number", customer.get("number"));
                customerMap.put("billStatus", true);
                customerMap.put("errors", "");
                resultList.add(customerMap);
            }
            map.put("successCount", String.valueOf(result.getSuccessPkIds().size()));
            map.put("failCount", "0");
            map.put("totalCount", String.valueOf(result.getSuccessPkIds().size()));
            map.put("result", resultList);
            return CustomApiResult.success(map);
        } else {
            DynamicObjectCollection successList = QueryServiceHelper.query("rm_mkt_custom", "id,number",
                    new QFilter("id", QCP.in, result.getSuccessPkIds()).toArray());
            for (DynamicObject customer : successList) {
                Map<String, Object> customerMap = new HashMap<>();
                customerMap.put("id", customer.get("id"));
                customerMap.put("number", customer.get("number"));
                customerMap.put("billStatus", true);
                customerMap.put("errors", "");
                resultList.add(customerMap);
            }
            map.put("successCount", String.valueOf(result.getSuccessPkIds().size()));
            map.put("failCount", String.valueOf(result.getValidateResult().getErrorDataIndexs().size()));
            map.put("totalCount", String.valueOf(result.getSuccessPkIds().size() + result.getValidateResult().getErrorDataIndexs().size()));
            List<OperateErrorInfo> allErrorInfo = result.getValidateResult().getValidateErrors().get(0).getAllErrorInfo();
            if (allErrorInfo.size() == 1 && !allErrorInfo.get(0).getMessage().contains("：")) {
                Map<String, Object> customerMap = new HashMap<>();
                customerMap.put("id", "");
                customerMap.put("number", result.getBillNos().toString().replace("}", "").split("=")[1]);
                customerMap.put("billStatus", false);
                customerMap.put("errors", allErrorInfo.get(0).getMessage());
                resultList.add(customerMap);
            } else {
                for (OperateErrorInfo operateErrorInfo : allErrorInfo) {
                    Map<String, Object> customerMap = new HashMap<>();
                    customerMap.put("id", "");
                    String[] split = operateErrorInfo.getMessage().split("：");
                    customerMap.put("number", split[0]);
                    customerMap.put("billStatus", false);
                    customerMap.put("errors", split[1]);
                    resultList.add(customerMap);
                }
                Map<String, Long> collect = resultList.stream()
                        .filter(o -> o.get("billStatus").equals(false))
                        .map(o -> o.get("number").toString())
                        .collect(Collectors.toList()).stream()
                        .collect(Collectors.groupingBy(Function.identity(), Collectors.counting()));
                List<String> duplicates = collect.entrySet().stream()
                        .filter(entry -> entry.getValue() > 1)
                        .map(Map.Entry::getKey)
                        .collect(Collectors.toList());
                List<Map<String, Object>> last = resultList.stream()
                        .filter(o -> o.get("billStatus").equals(false))
                        .filter(o -> duplicates.contains(o.get("number").toString()))
                        .collect(Collectors.toList());
                resultList.removeAll(last);
                for (String duplicate : duplicates) {
                    StringBuilder msg = new StringBuilder();
                    for (Map<String, Object> err : last) {
                        if (err.get("number").equals(duplicate)) {
                            msg.append(err.get("errors"));
                        }
                    }
                    Map<String, Object> customerMap = new HashMap<>();
                    customerMap.put("id", "");
                    customerMap.put("number", duplicate);
                    customerMap.put("billStatus", false);
                    customerMap.put("errors", msg);
                    resultList.add(customerMap);
                }
            }
            map.put("result", resultList);
            return CustomApiResult.success(map);
        }
    }

    public static Object handleBooleanFields(Object fields) {
        if (fields == null || fields.toString().isEmpty()) {
            return "0";
        } else {
            return fields;
        }
    }

}
